#Load libs
library(foreign)
library(MASS)
library(xtable)
library(mvtnorm)
library(arm)
library(stargazer)
library(mgcv)

#Load rep data set
data70 <- read.csv("/home/basti/svn/PhD/globalAnalysis/data/outcome/TheoryOfInsurgency-rep-data.csv")

##############
#Descriptives#
##############

#Generate correlation Matrix for main independent variables
matrix_vars <- c("geo_tbi","regime","support","power","lenerpc","occ","elev","dis","coldwar","mech","heli","numlang","ltradegdp")

corr_mat <- matrix(nrow=length(matrix_vars),ncol=length(matrix_vars))
colnames(corr_mat) <- matrix_vars
rownames(corr_mat) <- matrix_vars
for (i in 1:length(matrix_vars)){
	cvar <-  colnames(corr_mat)[i]
	cvals <- data70[,names(data70) == cvar]
	for(j in 1:i){
		rvar <-  rownames(corr_mat)[j]
		rvals <- data70[,names(data70) == rvar] 
		corr_mat[i,j] <- cor(cvals,rvals)
	
	}
}
xtable(round(corr_mat,3))

#Calcualte moments for the distributions of the explanatory variables
dist_vars <- matrix_vars
col_names <- c("Variable","Mean","Standard deviation","Minimum","Maximum","Observations","Definition")
dist_mat <- matrix(nrow=length(matrix_vars),ncol=length(col_names))
rownames(dist_mat) <- dist_vars
colnames(dist_mat) <- col_names
for(i in 1:length(dist_vars)){
	dist_mat[i,2] <-	mean(data70[,names(data70) == dist_vars[i]])	
	dist_mat[i,3] <-  sd(data70[,names(data70) == dist_vars[i]])
	dist_mat[i,4] <-  min(data70[,names(data70) == dist_vars[i]])
	dist_mat[i,5] <-  max(data70[,names(data70) == dist_vars[i]])
	dist_mat[i,6] <-  length(data70[,names(data70) == dist_vars[i]])
}
xtable(round(dist_mat,3))

#Plot bivariate relationship between TBI and casualties
par(mfrow=c(2,1))
plot(data70$geo_tbi,data70$casualties, main = "TBI and casualties for all conflicts",xlab="",ylab="Casualties")
tmp <- subset(data70, casualties <= 35000)
plot(tmp$geo_tbi,tmp$casualties,main = "TBI and casualties for less severe conflicts",xlab="",ylab="Casualties")


#################
#Analyze outcome#
#################

#Lyall model (modern)
model0 <- polr(as.ordered(ord_outcome)~ regime + support + power + lenerpc + occ + elev + coldwar + mech + heli + numlang + ltradegdp + log(pop_total)+ dis, data=data70, Hess = T)
#Mininal model
model1 <- polr(as.ordered(ord_outcome)~ dis + geo_tbi, data=data70, Hess = T)
#Full model
model2 <- polr(as.ordered(ord_outcome)~ regime + support + power + lenerpc + occ + elev + dis + coldwar + mech + heli + numlang + ltradegdp + log(pop_total) + geo_tbi , data=data70, Hess = T)
#Note that "dis" is causing quasi complete seperation
model3 <- polr(as.ordered(ord_outcome)~ regime + support + power + lenerpc + occ + elev + coldwar + mech + heli + numlang + ltradegdp + log(pop_total) + geo_tbi, data=data70, Hess = T)
#Good model
model4 <- polr(as.ordered(ord_outcome)~ regime + support + power + occ + coldwar + heli + dis + numlang + ltradegdp + geo_tbi, data70, Hess = T)

############
#Casualties#
############

#Some models require higher than default IWLS interations to converge (see "maxit" parameter)

#Baseline
model5 <- glm.nb(casualties ~ regime + support + power + lenerpc + occ + elev + dis + coldwar + mech + heli + numlang + ltradegdp + log(pop_total), maxit=100, link=log,data=data70)
#Minimal
model6 <- glm.nb(casualties ~ geo_tbi + I(geo_tbi^2), maxit=100, link=log, data=data70)
#Full
model7 <- glm.nb(casualties ~ regime + support + power + lenerpc + occ + elev + dis + coldwar + mech + heli + numlang + ltradegdp + log(pop_total) + geo_tbi + I(geo_tbi^2), maxit=100, link=log, data=data70)
#Good fit
model8 <-  glm.nb(formula = casualties ~ regime + support + occ + coldwar + mech + ltradegdp + geo_tbi + I(geo_tbi^2), data = data70, maxit = 100,  link = log)

#Generate latex regression table
stargazer(model0,model1,model2,model3,model4,model5,model6,model7,model8)

##################
#Simulate Effects#
##################
#Using adapted procedures from Elizabeth Menninga (http://menninga.web.unc.edu/files/2013/08/R_intro.pdf)

#Outcome | INSURGENT VICTORY |
fit <- model4
sims <- 1000 # For confidence intervals, we will simulate the model 1000 times
pp.sim <- rmvnorm(sims, coef(fit),vcov(fit)[1:length(coef(fit)),1:length(coef(fit))])
calc <- 100 # Number of times the predicted probability will be calculated.
ruler <- seq(0, 1, length = calc)
pe1 <- numeric(calc) # Empty vectors to store the point estimates and confidence intervals for when z = 1
lo1 <- numeric(calc)
hi1 <- numeric(calc)

tmp_model <- fit
sim_data <- data70[1,]
for(i in 1:calc){ # First for loop goes across the x-axis values (ruler).
        pp1 <- numeric(sims) # Empty vector to store the actual calculations at each point
       for(j in 1:sims){ # Now loop through all the simulated coefficients and each time calculate the predicted probability
                tmp_model$coefficients <- pp.sim[j,]
                sim_data$geo_tbi <- i / calc
                pp1[j] <- predict(tmp_model,type="p",newdata = sim_data)[3]#Corresponding to insurgent victory
       } # End calculation loop
        pe1[i] <- mean(pp1) # Point estimate on the probability scale.
        lo1[i] <- quantile(pp1, .025)# CIs on the probability scale.
        hi1[i] <- quantile(pp1, .975)
} # End x-axis loop (ruler)

plot(ruler, pe1, type = "l", lwd = 2, col = "black", xlim = c(min(data70$geo_tbi), max(data70$geo_tbi)),ylim = c(0,1), axes = T, xlab = "", ylab = "")
        lines(ruler, lo1, lwd = 2, lty = 2, col = "black")
        lines(ruler, hi1, lwd = 2, lty = 2, col = "black")
        title(ylab = "Predicted probability of insurgent victory", line = 3, cex.lab = 1.25)
        title(xlab = "TBI values", line = 2.75, cex.lab = 1.25)
        box()
        rug(data70$hc_geo_tbi, ticksize = .015)
        grid(col = "gray70")
        abline(h = seq(.1, .9, .2), col = "gray70", lty = 3)
        title(main = "The effect of TBI on the probability of insurgent victory", line = 2)


#Outcome | INCUMBENT VICTORY |
fit <- model4
sims <- 1000 # For confidence intervals, we will simulate the model 1000 times
pp.sim <- rmvnorm(sims, coef(fit),vcov(fit)[1:length(coef(fit)),1:length(coef(fit))])
calc <- 100 # Number of times the predicted probability will be calculated.
ruler <- seq(0, 1, length = calc)
pe1 <- numeric(calc) # Empty vectors to store the point estimates and confidence intervals for when z = 1
lo1 <- numeric(calc)
hi1 <- numeric(calc)

tmp_model <- fit
sim_data <- data70[1,]
for(i in 1:calc){ # First for loop goes across the x-axis values (ruler).
        pp1 <- numeric(sims) # Empty vector to store the actual calculations at each point
       for(j in 1:sims){ # Now loop through all the simulated coefficients and each time calculate the predicted probability
                tmp_model$coefficients <- pp.sim[j,]
                sim_data$geo_tbi <- i / calc
                pp1[j] <- predict(tmp_model,type="p",newdata = sim_data)[1]#Corresponding to incumbent victory
       } # End calculation loop
        pe1[i] <- mean(pp1) # Point estimate on the probability scale.
        lo1[i] <- quantile(pp1, .025)# CIs on the probability scale.
        hi1[i] <- quantile(pp1, .975)
} # End x-axis loop (ruler)

plot(ruler, pe1, type = "l", lwd = 2, col = "black", xlim = c(min(data70$geo_tbi), max(data70$geo_tbi)),ylim = c(0,1), axes = T, xlab = "", ylab = "")
        lines(ruler, lo1, lwd = 2, lty = 2, col = "black")
        lines(ruler, hi1, lwd = 2, lty = 2, col = "black")
        title(ylab = "Predicted probability of incumbent victory", line = 3, cex.lab = 1.25)
        title(xlab = "TBI values", line = 2.75, cex.lab = 1.25)
        box()
        rug(data70$hc_geo_tbi, ticksize = .015)
        grid(col = "gray70")
        abline(h = seq(.1, .9, .2), col = "gray70", lty = 3)
        title(main = "The effect of TBI on the probability of incumbent victory", line = 2)


#Casualties
fit <- model8
sims <- 1000 # For confidence intervals, we will simulate the model 1000 times
pp.sim <- rmvnorm(sims, coef(fit),vcov(fit))#[2:length(vcov(fit)[1,]),2:length(vcov(fit)[1,])])
calc <- 100 # Number of times the predicted probability will be calculated.
ruler <- seq(0, 1, length = calc)
pe1 <- numeric(calc) # Empty vectors to store the point estimates and confidence intervals for when z = 1
lo1 <- numeric(calc)
hi1 <- numeric(calc)

tmp_model <- fit
sim_data <- data70[1,]
for(i in 1:calc){ # First for loop goes across the x-axis values (ruler).
        pp1 <- numeric(sims) # Empty vector to store the actual calculations at each point
        for(j in 1:sims){ # Now loop through all the simulated coefficients and each time calculate the predicted probability
                tmp_model$coefficients <- pp.sim[j,]
                sim_data$geo_tbi <- i / calc
                #pp1[j] <- predict(tmp_model,type="p",newdata = sim_data)[3]
                pp1[j] <- exp(predict(tmp_model,newdata = sim_data))
        } # End calculation loop
        pe1[i] <- mean(pp1) # Point estimate on the probability scale.
        lo1[i] <- quantile(pp1, .025)# CIs on the probability scale.
        hi1[i] <- quantile(pp1, .975)
} # End x-axis loop (ruler)

# Now we plot the observed range of partisan (ruler) on the x-axis and the point estimate and confidence intervals
plot(ruler, pe1, xlim = c(min(data70$geo_tbi), max(data70$geo_tbi)), ylim = c(0,max(hi1)),type = "l", lwd = 2, col = "black", xlab = "", ylab = "")
        lines(ruler, lo1, lwd = 2, lty = 2, col = "black")
        lines(ruler, hi1, lwd = 2, lty = 2, col = "black")
        title(ylab = "Predicted casualties", line = 3, cex.lab = 1.25)
        title(xlab = "TBI values", line = 2.75, cex.lab = 1.25)
        box()
        rug(data70$hc_geo_tbi, ticksize = .015)
        grid(col = "gray70")
        abline(h = seq(.1, .9, .2), col = "gray70", lty = 3)
        title(main = "The effect of TBI on predicted casualties", line = 2)



############
#Prediction#
############

#Outcome
#in-sample 
paste("m0:",sum(abs(as.numeric(predict(model0)) - as.numeric(data70$ord_outcome))))
paste("m1:",sum(abs(as.numeric(predict(model1)) - as.numeric(data70$ord_outcome))))
paste("m2:",sum(abs(as.numeric(predict(model2)) - as.numeric(data70$ord_outcome))))
paste("m3:",sum(abs(as.numeric(predict(model3)) - as.numeric(data70$ord_outcome))))
paste("m4:",sum(abs(as.numeric(predict(model4)) - as.numeric(data70$ord_outcome))))
#paste("m5:",sum(abs(as.numeric(predict(model5)) - as.numeric(data70$ord_outcome))))

#Casualties
pc_m5 <- exp(predict(model5))
pc_m6 <- exp(predict(model6))
pc_m7 <- exp(predict(model7))
pc_m8 <- exp(predict(model8))

cas_data <- subset(data70,!is.na(casualties))
sum(abs(pc_m5 - as.numeric(cas_data$casualties)) / length(cas_data$casualties))
sum(abs(pc_m6 - as.numeric(cas_data$casualties)) / length(cas_data$casualties))
sum(abs(pc_m7 - as.numeric(cas_data$casualties)) / length(cas_data$casualties))
sum(abs(pc_m8 - as.numeric(cas_data$casualties)) / length(cas_data$casualties))


#out-of-sample
check_cv <- function(fit,data)
{
        #Absolute value of ordinal missprediction as CV score
        get_cv_score <- function(fit,data,k){
                formula <- fit$call
                observed <- data[k,]
                data <- data[-k,]
                pred_fit <- polr(fit$call,data)
                predicted <- predict(pred_fit,newdata=observed)
                return(abs(as.numeric(predicted) - as.numeric(observed$ord_outcome)))
        }
        misspred <- 0
        for (i in 1:length(data$ord_outcome)){
                misspred <- misspred + get_cv_score(fit,data,i)
        }
        return(misspred)
}
check_cv(model0,data70)
#[1] 39
check_cv(model1,data70)
#[1] 34
check_cv(model2,data70)
#[1] 40
check_cv(model3,data70)
#[1] 39
check_cv(model4,data70)
#[1] 37

#Casualties
check_cv_cas <- function(fit,data){
        #Absolute value of ordinal missprediction as CV score
        get_cv_score <- function(fit,data,k){
                formula <- fit$call
                observed <- data[k,]
                data <- data[-k,]
                #tryCatch({
                #cat("fitting...")
                pred_fit <- glm.nb(fit$call,data, maxit = 100)
                #pc <- glm.nb(fit$call,data) 
                #cat("predicting...")
                pc <- exp(predict(pred_fit,newdata = observed))
                #cat("\n")
                return(sum(abs(as.numeric(pc) - as.numeric(observed$casualties))))

                                #pp <- dnbinom(0, mu = fitted(pred_fit), size = pred_fit$theta)
                #predicted <- apply(pp, 1, which.max) - 1
                #})
        }
        misspred <- rep(NA,length(data$casualties))
        for (i in 1:length(data$ord_outcome)){
                try(misspred[i] <- get_cv_score(fit,data,i))
                #cat("sum dev: ",misspred,"\n")
        }
        #print(misspred)
        misspred <- subset(misspred,!is.na(misspred))
        return(sum(misspred)/length(misspred[!is.na(misspred)]))
}
check_cv_cas(model5,cas_data)
#[1] 81547.37
check_cv_cas(model6,cas_data)
#[1] 49023.38
check_cv_cas(model7,cas_data)
#[1] 61568.83
check_cv_cas(model8,cas_data)
#[1] 48518.98

###################
#Residual analysis#
###################
outcome_deviation <- abs (as.numeric(predict(model4)) - as.numeric(data70$ord_outcome))
data70$war[outcome_deviation == max(outcome_deviation)]
#[1] Cambodia v. FUNK

residuals  <- as.data.frame(cbind(as.character(subset(data70$incumb,!is.na(data70$casualties))),as.character(subset(data70$insurg,!is.na(data70$casualties))) ,as.character(subset(data70$yearbg,!is.na(data70$casualties))),as.character(subset(data70$yearend,!is.na(data70$casualties))),as.numeric(subset(data70,!is.na(casualties))$casualties - (as.numeric(exp(predict(model8))))), as.numeric(subset(data70,!is.na(casualties))$ord_outcome) - as.numeric(predict(model4))[!is.na(data70$casualties)]))

names (residuals) <- c("Incumbent","Insurgent","Start","End","Casualties","Outcome")
residuals$Casualties <- as.integer(round(as.numeric(as.character(residuals$Casualties)), 0))
residuals$Outcome <- as.integer(as.character(residuals$Outcome))
residuals[order(residuals$Outcome),]
residuals <- residuals[order(residuals$Casualties),]
rownames(residuals) <- (seq(1,length(residuals[,1]),1))
xtable(residuals)


######################################################
#Second remoteness operationalization: Urban Distance#
######################################################

tmp <- subset(data70,!is.na(urban_tbi))
#Indicators are positively correlated
plot(tmp$geo_tbi,tmp$urban_tbi)
cor(tmp$geo_tbi,tmp$urban_tbi)

#mininal model
urban_model1 <- polr(as.ordered(ord_outcome)~ dis + urban_tbi, data=data70, Hess = T)
#Full model
urban_model2 <- polr(as.ordered(ord_outcome)~ regime + support + power + lenerpc + occ + elev + coldwar + mech + heli + numlang + ltradegdp + log(pop_total) + dis + urban_tbi, data=data70, Hess = T)
#Note that "dis" is causing quasi complete seperation
urban_model3 <- polr(as.ordered(ord_outcome)~ regime + support + power + lenerpc + occ + elev + coldwar + mech + heli + numlang + ltradegdp + log(pop_total) + urban_tbi, data=data70, Hess = T)
#Good model
urban_model4 <- polr(as.ordered(ord_outcome)~ regime + support + power + occ + coldwar + heli + dis + numlang + ltradegdp + urban_tbi, data70, Hess = T)

check_cv(urban_model1,tmp)
#[1] 36
check_cv(urban_model2,tmp)
#[1] 41
check_cv(urban_model3,tmp)
#[1] 40
check_cv(urban_model4,tmp)
#[1] 37

############
#Casualties#
############

#Minimal
urban_model6 <- glm.nb(casualties ~ urban_tbi + I(urban_tbi^2), maxit=100, link=log, data=data70)
#Full
urban_model7 <- glm.nb(casualties ~ regime + support + power + lenerpc + occ + elev + dis + coldwar + mech + heli + numlang + ltradegdp + log(pop_total) + urban_tbi + I(urban_tbi^2), maxit=100, link=log, data=data70)
#Good fit
urban_model8 <-  glm.nb(formula = casualties ~ regime + support + occ + coldwar + mech + ltradegdp + urban_tbi + I(urban_tbi^2), data = data70, maxit = 100,  link = log)

stargazer(urban_model1,urban_model2,urban_model3,urban_model4,urban_model6,urban_model7,urban_model8)

#########################
#Gam Estimate Casulaties#
#########################

par(mfrow=c(1,2))
model_cas_gam <- gam(casualties~ dis+ s(geo_tbi), data=data70,method="ML",family=negbin(c(1,10)),link="log")
plot(model_cas_gam,main="Smooth estimate for TBI\n on casualties",ylab="estimate",xlab="TBI")
abline(h=0,col="red")
model_cas_gam2 <- gam( casualties ~ regime + support + occ + coldwar + mech + ltradegdp + s(geo_tbi), family="negbin(c(1,10))",method="ML",optimizer="perf", data = data70,link="log")
plot(model_cas_gam2,main="Smooth estimate for TBI\n on casualties",ylab="estimate",xlab="TBI")
abline(h=0,col="red")

########################
#Binary dependet models#
########################

incumbent_vic <- glm(win ~ regime + support + power + lenerpc + occ + dis + coldwar + heli + numlang +ltradegdp ,data = data70,family = binomial())
incumbent_vic2 <- glm(win ~ regime + support + power + lenerpc + occ + dis + coldwar + heli + numlang +ltradegdp + log(pop_total) + geo_tbi,data = data70,family = binomial())
incumbent_def <- glm(defeat ~ regime + support + power + lenerpc + occ + dis + coldwar + heli +  numlang +ltradegdp, data = data70,family = binomial())
incumbent_def2 <- glm(defeat ~ regime + support + power + lenerpc + occ + dis + coldwar + heli +  numlang +ltradegdp + log(pop_total) +geo_tbi,data = data70,family = binomial())

stargazer(incumbent_vic,incumbent_vic2,incumbent_def,incumbent_def2)

################################
#Omit power and heli           #
#as they are correlated at 0.62#
################################
model2_no_power <- polr(as.ordered(ord_outcome)~ regime + support + lenerpc + occ + elev + dis + coldwar + mech + numlang + ltradegdp + log(pop_total) + geo_tbi + heli, data=data70, Hess = T)
model2_no_heli <- polr(as.ordered(ord_outcome)~ regime + support + lenerpc + occ + elev + dis + coldwar + mech + numlang + ltradegdp + log(pop_total) + geo_tbi + power, data=data70, Hess = T)
model7_no_power <- glm.nb(casualties ~ regime + support + lenerpc + occ + elev + dis + coldwar + mech + numlang + ltradegdp + log(pop_total) + geo_tbi + I(geo_tbi^2) + heli, maxit=100, link=log, data=data70)
model7_no_heli <- glm.nb(casualties ~ regime + support + lenerpc + occ + elev + dis + coldwar + mech + numlang + ltradegdp + log(pop_total) + geo_tbi + I(geo_tbi^2) + power, maxit=100, link=log, data=data70)

stargazer(model2_no_power, model2_no_heli, model7_no_power, model7_no_heli)

####################################################################
#Omit support as it is possibly endogenous to the conflict process #
####################################################################

#########
#Outcome#
#########

#Lyall model (modern)
model0 <- polr(as.ordered(ord_outcome)~ regime +  power + lenerpc + occ + elev + coldwar + mech + heli + numlang + ltradegdp + log(pop_total)+ dis, data=data70, Hess = T)
#Mininal model
model1 <- polr(as.ordered(ord_outcome)~ dis + geo_tbi, data=data70, Hess = T)
#Full model
model2 <- polr(as.ordered(ord_outcome)~ regime + power + lenerpc + occ + elev + dis + coldwar + mech + heli + numlang + ltradegdp + log(pop_total) + geo_tbi , data=data70, Hess = T)
#Note that "dis" is causing quasi complete seperation
model3 <- polr(as.ordered(ord_outcome)~ regime + power + lenerpc + occ + elev + coldwar + mech + heli + numlang + ltradegdp + log(pop_total) + geo_tbi, data=data70, Hess = T)
#Good model
model4 <- polr(as.ordered(ord_outcome)~ regime + power + occ + coldwar + heli + dis + numlang + ltradegdp + geo_tbi, data70, Hess = T)

############
#Casualties#
############

#Baseline
model5 <- glm.nb(casualties ~ regime + power + lenerpc + occ + elev + dis + coldwar + mech + heli + numlang + ltradegdp + log(pop_total), maxit=100, link=log,data=data70)
#Minimal
model6 <- glm.nb(casualties ~ geo_tbi + I(geo_tbi^2), maxit=100, link=log, data=data70)
#Full
model7 <- glm.nb(casualties ~ regime + power + lenerpc + occ + elev + dis + coldwar + mech + heli + numlang + ltradegdp + log(pop_total) + geo_tbi + I(geo_tbi^2), maxit=100, link=log, data=data70)
#Good one
model8 <-  glm.nb(formula = casualties ~ regime + occ + coldwar + mech + ltradegdp + geo_tbi + I(geo_tbi^2), data = data70, maxit = 100,  link = log)

stargazer(model0,model1,model2,model3,model4,model5,model6,model7,model8)

